/*
	Utility to build a Gap fill exercise html file
	See: https://direns.mines-paristech.fr/Sites/Thopt/en/co/module_Logiciel_3.html
	
	Copyright Renaud Gicquel 2023
	This document is licensed under CC BY-SA 4.0 (Attribution-ShareAlike 4.0 International)
	
*/

package rg.tat;

import javax.swing.*;

import org.jsoup.Jsoup;
import org.jsoup.nodes.Document;
import org.jsoup.nodes.Element;
import org.jsoup.select.Elements;

import java.io.BufferedWriter;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.ResourceBundle;
import java.util.StringTokenizer;
import java.util.Vector;

public class GapFillHtmlBuilder extends JFrame {

	static ResourceBundle res = ResourceBundle.getBundle("rg.tat.Res");

	JMenuBar menuBar = new JMenuBar();
    JMenu fileMenu = new JMenu(res.getString("File"));
    JMenuItem openItem = new JMenuItem(res.getString("Open_HTML_file"));
    JMenuItem saveItem = new JMenuItem(res.getString("Save"));
    JMenuItem quitMenuItem = new JMenuItem(res.getString("Quit"));
    JMenu aboutMenu = new JMenu(res.getString("About"));
    JMenuItem aboutMenuItem = new JMenuItem(res.getString("About_GapFill"));

    private JTextArea origTextArea;
    private JTextArea htmlTextArea;
    private JTextField titleField;
    private JTextField consigneField;
    private JLabel titleLabel;
    private JLabel consigneLabel;
    Vector<String> texteConnu = new Vector<>();
    Vector<String> trous = new Vector<>();
    Document document;
    /*
     * Voir problème codage utf-8
     *
     * 
     */
    
    public GapFillHtmlBuilder() {
        setTitle(res.getString("HTML_Builder"));
        setSize(800, 630);
        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

        origTextArea = new JTextArea();
        origTextArea.setWrapStyleWord(true);
        origTextArea.setLineWrap(true);
        htmlTextArea = new JTextArea();
        htmlTextArea.setLineWrap(true);
        htmlTextArea.setWrapStyleWord(true);

        JScrollPane scrollPane1 = new JScrollPane(origTextArea);
        scrollPane1.setBounds(0, 0, 750, 150);
        JScrollPane scrollPane2 = new JScrollPane(htmlTextArea);
        scrollPane2.setBounds(0, 150, 750, 310);

        JButton parseButton = new JButton(res.getString("Generer_HTML"));
        parseButton.setBounds(180, 5, 123, 25);
        parseButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                generateHtml();
            }
        });
        getContentPane().setLayout(null);
  	  	HyperlinkLabel hyperlinkLabel = new HyperlinkLabel(res.getString("check_syntax"),"https://validator.nu/#textarea");
	    hyperlinkLabel.setBounds(320, 5, 300, 25);
	    hyperlinkLabel.setFont(new java.awt.Font("Dialog", 5, 17));

        JPanel mainPanel = new JPanel();
        mainPanel.setBounds(10, 0, 782, 600);
        mainPanel.setLayout(null);
        
        JPanel textAreasPanel = new JPanel();
        textAreasPanel.setBounds(0, 50, 782, 460);
        textAreasPanel.setLayout(null);
        textAreasPanel.add(scrollPane1);
        textAreasPanel.add(scrollPane2);

        mainPanel.add(textAreasPanel);

        JPanel buttonPanel = new JPanel();
        buttonPanel.setBounds(0, 518, 782, 35);
        buttonPanel.setLayout(null);
        buttonPanel.add(parseButton);
        buttonPanel.add(hyperlinkLabel);
        mainPanel.add(buttonPanel);

        getContentPane().add(mainPanel);
        
        titleField = new JTextField();
        titleField.setBounds(87, 0, 663, 22);
        mainPanel.add(titleField);
        consigneLabel = new JLabel(res.getString("Consigne"));
        consigneLabel.setBounds(0, 25, 100, 22);
        mainPanel.add(consigneLabel);
        consigneField  = new JTextField();
        consigneField.setBounds(87, 25, 663, 22);
        mainPanel.add(consigneField);
        titleLabel = new JLabel(res.getString("Title"));
        titleLabel.setBounds(0, 0, 100, 22);
        mainPanel.add(titleLabel);
 
	    fileMenu.addSeparator();
	    fileMenu.add(quitMenuItem);
	    fileMenu.add(openItem);
	    fileMenu.add(saveItem);
	    aboutMenu.add(aboutMenuItem);
	    menuBar.add(fileMenu);
	    menuBar.add(aboutMenu);
	    this.setJMenuBar(menuBar);
	    
	    openItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_O, InputEvent.CTRL_DOWN_MASK));
	    openItem.addActionListener(new ActionListener() {
	        public void actionPerformed(ActionEvent e) {
	        	ouvrirFichierHtml();	            }
	        }
	    );
	    saveItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S, InputEvent.CTRL_DOWN_MASK));
	    saveItem.addActionListener(new ActionListener() {
	        public void actionPerformed(ActionEvent e) {
	        	sauvegarderContenu();
	        }
	    });

	    quitMenuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Q, InputEvent.CTRL_DOWN_MASK));
	    quitMenuItem.addActionListener(new ActionListener() {
	        public void actionPerformed(ActionEvent e) {
	            System.exit(0);
	        }
	    });

	    aboutMenuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_I, InputEvent.CTRL_DOWN_MASK));
	    aboutMenuItem.addActionListener(new ActionListener() {
	        public void actionPerformed(ActionEvent e) {
	            displayAbout();
	        }
	    });




        setVisible(true);
    }
    private void displayAbout() {
    	JOptionPane.showMessageDialog(null, 
    			res.getString("About_CC")
    			, res.getString("About"), JOptionPane.INFORMATION_MESSAGE);	    
    }


    private void ouvrirFichierHtml() {
        JFileChooser fileChooser = new JFileChooser(new File("distrib"));
        int result = fileChooser.showOpenDialog(this);

        if (result == JFileChooser.APPROVE_OPTION) {
            File selectedFile = fileChooser.getSelectedFile();
            Path filePath = Paths.get(selectedFile.getAbsolutePath());

            try {
                String content = new String(Files.readAllBytes(filePath));
                htmlTextArea.setText(content);
                String htmlContent = htmlTextArea.getText();
                Element doc = Jsoup.parse(htmlContent);
                Element titleElement = doc.select("h2").first();
                Element consigneElement = doc.select("h4").first();
                String title="";
                if (titleElement != null) {
                    title = titleElement.text();
                    titleField.setText(title);
                }
                if (consigneElement != null) {
                    title = consigneElement.text();
                    consigneField.setText(title);
                }

            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        origTextArea.setText(res.getString("Enonce_Trous"));
    }

    private void generateHtml() {
        String htmlContent = htmlTextArea.getText();
        document = Jsoup.parse(htmlContent);
        updateTitle();
    	updateFormTag();
    	updateReponsesExactes();
    	updateReponsesPossibles();
    	htmlTextArea.setText(document.html());
    }
    private void updateReponsesPossibles() {
        // Convertir le Vector<String> en une représentation JSON

        	Element scriptElement=document.select("script").first();
        	 String javascriptCode=scriptElement.html();
             String startMarker = "const reponsesPossibles = [";
             String endMarker = "];";

             int startIndex = javascriptCode.indexOf(startMarker);
             int endIndex = javascriptCode.indexOf(endMarker, startIndex);
             String debut=javascriptCode.substring(0, startIndex);
             String fin=javascriptCode.substring(endIndex, javascriptCode.length());
             for(int i=0;i<trous.size()-1;i++){
            	 startMarker=startMarker+"\""+trous.elementAt(i)+"\",";      
             }
        	 String milieu=startMarker+"\""+trous.elementAt(trous.size()-1)+"\"\n";      
//             System.out.println("reponsesPossibles\n"+startMarker);
             String reponsesPossibles = debut+milieu+fin;
             scriptElement.html(reponsesPossibles);



    }
    private void updateReponsesExactes() {
        // Convertir le Vector<String> en une représentation JSON
  //      String jsonRepresentation = convertVectorToJson(trous);

        // Incorporer le JSON dans le code JavaScript
   //     String codeJavaScript = embedJsonInJavaScript(jsonRepresentation);

        	Element scriptElement=document.select("script").first();
        	 String javascriptCode=scriptElement.html();
             String startMarker = "const reponsesExactes = [";
             String endMarker = "];";

             int startIndex = javascriptCode.indexOf(startMarker);
             int endIndex = javascriptCode.indexOf(endMarker, startIndex);
             String debut=javascriptCode.substring(0, startIndex);
             String fin=javascriptCode.substring(endIndex, javascriptCode.length());
             for(int i=0;i<trous.size()-1;i++){
            	 startMarker=startMarker+"[\""+trous.elementAt(i)+"\"],\n";      
             }
        	 String milieu=startMarker+"\""+trous.elementAt(trous.size()-1)+"\"\n";      
//           System.out.println("reponsesPossibles\n"+startMarker);
             String newReponsesExactes = debut+milieu+fin;
             scriptElement.html(newReponsesExactes);

    }   
    
    
    private void updateTitle() {

	    // Met à jour le titre et la consigne
	    Element titleElement = document.select("h2").first();
	    titleElement.html(titleField.getText());
	    Element consigneElement = document.select("h4").first();
	    consigneElement.html(consigneField.getText());
    }

    private void updateFormTag() {
        
        String origContent = origTextArea.getText();
        
        StringTokenizer st=new StringTokenizer(origContent,"*");
 //       System.out.println("debut texte à trous");
        while(st.hasMoreTokens()){
	        String tokenOK= st.nextToken();
	         if(st.hasMoreTokens()){   
	        texteConnu.addElement(tokenOK);
	        String tokenTrou= st.nextToken();
	        trous.addElement(tokenTrou);
	         }
        }
/*        System.out.println("texteConnu");
        for(int i=0;i<texteConnu.size();i++){
	        System.out.println(texteConnu.elementAt(i));      
        }

        System.out.println("trous");
        for(int i=0;i<trous.size();i++){
	        System.out.println(trous.elementAt(i));      
        }
        */
        //Création de la balise  form 
        
         String baliseForm="";

        for(int i=0;i<texteConnu.size();i++){
        	baliseForm = baliseForm+texteConnu.elementAt(i)
        			+"<select class=\"gapSelect\"></select>";      
        }
        String baliseForm2=wrapLinesWithPTags(baliseForm);
 //       System.out.println("baliseForm2\n"+baliseForm2); 
        
        baliseForm2 = "<form id=\"gapFillForm\">\n"+baliseForm2+"</p>\n<button type=\"button\" onclick=\"verifier()\">Vérifier</button>\n </form>";
 //       System.out.println("baliseForm2 complet\n"+baliseForm2); 
               
        Element newForm=new Element(baliseForm2);
 //       System.out.println("newForm\n"+newForm.html()); 

        // On récupère le contenu précédent de la balise form
        Elements mainElements = document.select("form");
        //  On le remplace par le nouveau contenu
       if (mainElements != null && !mainElements.isEmpty()) {
             mainElements.first().html(baliseForm2);
            
        }
   }
    
    private static String wrapLinesWithPTags(String text) {
        StringBuilder result = new StringBuilder();
        String[] lines = text.split("\n");

        for (String line : lines) {
            result.append("<p>").append(line.trim()).append("</p>").append("\n");
        }

        return result.toString();
    }


    private String extractReponsesExactes(String javascriptCode) {
        String startMarker = "const reponsesExactes = [";
        String endMarker = "];";
        String h = "";
        int startIndex = javascriptCode.indexOf(startMarker);
        int endIndex = javascriptCode.indexOf(endMarker, startIndex);

        if (startIndex != -1 && endIndex != -1) {
            String reponsesExactesContent = javascriptCode.substring(startIndex + startMarker.length(), endIndex + 1);
 //           System.out.println(reponsesExactesContent);
            h = h + reponsesExactesContent;
        } else {
 //           System.out.println("reponsesExactes non trouvé.");
        }
        return h;
    }
    private void sauvegarderContenu() {
        JFileChooser fileChooser = new JFileChooser(new File("distrib"));
        int result = fileChooser.showSaveDialog(this);

        if (result == JFileChooser.APPROVE_OPTION) {
            File selectedFile = fileChooser.getSelectedFile();
            Path filePath = Paths.get(selectedFile.getAbsolutePath());
            // Check if the selected file already exists
            if (selectedFile.exists()) {
                int confirmResult = JOptionPane.showConfirmDialog(
                    this,
                    res.getString("File_exists_replace"),
                    res.getString("Replace_confirmation"),
                    JOptionPane.YES_NO_OPTION
                );

                if (confirmResult != JOptionPane.YES_OPTION) {
                    return;
                }
            }          

            try (BufferedWriter writer = new BufferedWriter(new FileWriter(filePath.toFile()))) {
                // Écrire le contenu de textArea1 dans le fichier
                writer.write(htmlTextArea.getText());
 //               JOptionPane.showMessageDialog(this, "Contenu sauvegardé avec succès", "Sauvegarde réussie", JOptionPane.INFORMATION_MESSAGE);
            } catch (IOException e) {
                e.printStackTrace();
 //               JOptionPane.showMessageDialog(this, "Erreur lors de la sauvegarde", "Erreur", JOptionPane.ERROR_MESSAGE);
            }
        }
        
    }
    public static void main(String[] args) {
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                new GapFillHtmlBuilder();
            }
        });
    }
}
