/*
	Utility to build a Gap fill exercise html file
	See: https://direns.mines-paristech.fr/Sites/Thopt/en/co/module_Logiciel_3.html
	
	Copyright Renaud Gicquel 2023
	This document is licensed under CC BY-SA 4.0 (Attribution-ShareAlike 4.0 International)
	
*/

package rg.tat;

import javax.swing.*;
import java.awt.*;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;

public class HyperlinkLabel extends JLabel {
    private URI uri;

    public HyperlinkLabel(String text, String uri) {
        super("<html><u>" + text + "</u></html>");
        setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));

        try {
            this.uri = new URI(uri);
        } catch (URISyntaxException e) {
            e.printStackTrace();
        }

        addMouseListener(new MouseAdapter() {
            @Override
            public void mouseClicked(MouseEvent e) {
                openWebPage(HyperlinkLabel.this.uri);
            }

            @Override
            public void mouseEntered(MouseEvent e) {
                setForeground(Color.BLUE);
            }

            @Override
            public void mouseExited(MouseEvent e) {
                setForeground(Color.BLACK);
            }
        });
    }

    private void openWebPage(URI uri) {
        if (Desktop.isDesktopSupported()) {
            Desktop desktop = Desktop.getDesktop();
            try {
                desktop.browse(uri);
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

}
